# flake8: noqa
# type: ignore

from seam.seam import Seam
from seam.types import SeamHttpApiError
from seam.seam_multi_workspace import SeamMultiWorkspace
from seam.options import SeamHttpInvalidOptionsError
from seam.auth import SeamHttpInvalidTokenError
from seam.routes.action_attempts import (
    SeamActionAttemptError,
    SeamActionAttemptFailedError,
    SeamActionAttemptTimeoutError,
)



from seam.types import AbstractSeam as Seam
from seam.routes.types import AbstractPhones, Phone
from typing import Optional, Any, List, Dict, Union
from seam.routes.phones_simulate import PhonesSimulate


class Phones(AbstractPhones):
    seam: Seam

    def __init__(self, seam: Seam):
        self.seam = seam
        self._simulate = PhonesSimulate(seam=seam)

    @property
    def simulate(self) -> PhonesSimulate:
        return self._simulate

    def deactivate(self, *, device_id: str) -> None:
        json_payload = {}

        if device_id is not None:
            json_payload["device_id"] = device_id

        self.seam.client.post("/phones/deactivate", json=json_payload)

        return None

    def list(self, *, owner_user_identity_id: Optional[str] = None) -> List[Phone]:
        json_payload = {}

        if owner_user_identity_id is not None:
            json_payload["owner_user_identity_id"] = owner_user_identity_id

        res = self.seam.client.post("/phones/list", json=json_payload)

        return [Phone.from_dict(item) for item in res["phones"]]



from seam.types import AbstractSeam as Seam
from seam.routes.types import AbstractAcsEntrances, AcsEntrance, AcsCredential
from typing import Optional, Any, List, Dict, Union


class AcsEntrances(AbstractAcsEntrances):
    seam: Seam

    def __init__(self, seam: Seam):
        self.seam = seam

    def get(self, *, acs_entrance_id: str) -> AcsEntrance:
        json_payload = {}

        if acs_entrance_id is not None:
            json_payload["acs_entrance_id"] = acs_entrance_id

        res = self.seam.client.post("/acs/entrances/get", json=json_payload)

        return AcsEntrance.from_dict(res["acs_entrance"])

    def grant_access(self, *, acs_entrance_id: str, acs_user_id: str) -> None:
        json_payload = {}

        if acs_entrance_id is not None:
            json_payload["acs_entrance_id"] = acs_entrance_id
        if acs_user_id is not None:
            json_payload["acs_user_id"] = acs_user_id

        self.seam.client.post("/acs/entrances/grant_access", json=json_payload)

        return None

    def list(
        self,
        *,
        acs_credential_id: Optional[str] = None,
        acs_system_id: Optional[str] = None
    ) -> List[AcsEntrance]:
        json_payload = {}

        if acs_credential_id is not None:
            json_payload["acs_credential_id"] = acs_credential_id
        if acs_system_id is not None:
            json_payload["acs_system_id"] = acs_system_id

        res = self.seam.client.post("/acs/entrances/list", json=json_payload)

        return [AcsEntrance.from_dict(item) for item in res["acs_entrances"]]

    def list_credentials_with_access(
        self, *, acs_entrance_id: str, include_if: Optional[List[str]] = None
    ) -> List[AcsCredential]:
        json_payload = {}

        if acs_entrance_id is not None:
            json_payload["acs_entrance_id"] = acs_entrance_id
        if include_if is not None:
            json_payload["include_if"] = include_if

        res = self.seam.client.post(
            "/acs/entrances/list_credentials_with_access", json=json_payload
        )

        return [AcsCredential.from_dict(item) for item in res["acs_credentials"]]



from seam.types import AbstractSeam as Seam
from seam.routes.types import AbstractNoiseSensorsSimulate
from typing import Optional, Any, List, Dict, Union


class NoiseSensorsSimulate(AbstractNoiseSensorsSimulate):
    seam: Seam

    def __init__(self, seam: Seam):
        self.seam = seam

    def trigger_noise_threshold(self, *, device_id: str) -> None:
        json_payload = {}

        if device_id is not None:
            json_payload["device_id"] = device_id

        self.seam.client.post(
            "/noise_sensors/simulate/trigger_noise_threshold", json=json_payload
        )

        return None



from seam.types import AbstractSeam as Seam
from seam.routes.types import AbstractLocks, Device, ActionAttempt
from typing import Optional, Any, List, Dict, Union


class Locks(AbstractLocks):
    seam: Seam

    def __init__(self, seam: Seam):
        self.seam = seam

    def get(
        self, *, device_id: Optional[str] = None, name: Optional[str] = None
    ) -> Device:
        json_payload = {}

        if device_id is not None:
            json_payload["device_id"] = device_id
        if name is not None:
            json_payload["name"] = name

        res = self.seam.client.post("/locks/get", json=json_payload)

        return Device.from_dict(res["device"])

    def list(
        self,
        *,
        connect_webview_id: Optional[str] = None,
        connected_account_id: Optional[str] = None,
        connected_account_ids: Optional[List[str]] = None,
        created_before: Optional[str] = None,
        custom_metadata_has: Optional[Dict[str, Any]] = None,
        device_ids: Optional[List[str]] = None,
        device_type: Optional[str] = None,
        device_types: Optional[List[str]] = None,
        exclude_if: Optional[List[str]] = None,
        include_if: Optional[List[str]] = None,
        limit: Optional[float] = None,
        manufacturer: Optional[str] = None,
        user_identifier_key: Optional[str] = None
    ) -> List[Device]:
        json_payload = {}

        if connect_webview_id is not None:
            json_payload["connect_webview_id"] = connect_webview_id
        if connected_account_id is not None:
            json_payload["connected_account_id"] = connected_account_id
        if connected_account_ids is not None:
            json_payload["connected_account_ids"] = connected_account_ids
        if created_before is not None:
            json_payload["created_before"] = created_before
        if custom_metadata_has is not None:
            json_payload["custom_metadata_has"] = custom_metadata_has
        if device_ids is not None:
            json_payload["device_ids"] = device_ids
        if device_type is not None:
            json_payload["device_type"] = device_type
        if device_types is not None:
            json_payload["device_types"] = device_types
        if exclude_if is not None:
            json_payload["exclude_if"] = exclude_if
        if include_if is not None:
            json_payload["include_if"] = include_if
        if limit is not None:
            json_payload["limit"] = limit
        if manufacturer is not None:
            json_payload["manufacturer"] = manufacturer
        if user_identifier_key is not None:
            json_payload["user_identifier_key"] = user_identifier_key

        res = self.seam.client.post("/locks/list", json=json_payload)

        return [Device.from_dict(item) for item in res["devices"]]

    def lock_door(
        self,
        *,
        device_id: str,
        sync: Optional[bool] = None,
        wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]] = None
    ) -> ActionAttempt:
        json_payload = {}

        if device_id is not None:
            json_payload["device_id"] = device_id
        if sync is not None:
            json_payload["sync"] = sync

        res = self.seam.client.post("/locks/lock_door", json=json_payload)

        return self.seam.action_attempts.decide_and_wait(
            action_attempt=ActionAttempt.from_dict(res["action_attempt"]),
            wait_for_action_attempt=wait_for_action_attempt,
        )

    def unlock_door(
        self,
        *,
        device_id: str,
        sync: Optional[bool] = None,
        wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]] = None
    ) -> ActionAttempt:
        json_payload = {}

        if device_id is not None:
            json_payload["device_id"] = device_id
        if sync is not None:
            json_payload["sync"] = sync

        res = self.seam.client.post("/locks/unlock_door", json=json_payload)

        return self.seam.action_attempts.decide_and_wait(
            action_attempt=ActionAttempt.from_dict(res["action_attempt"]),
            wait_for_action_attempt=wait_for_action_attempt,
        )



from seam.types import AbstractSeam as Seam
from seam.routes.types import AbstractDevicesSimulate
from typing import Optional, Any, List, Dict, Union


class DevicesSimulate(AbstractDevicesSimulate):
    seam: Seam

    def __init__(self, seam: Seam):
        self.seam = seam

    def remove(self, *, device_id: str) -> None:
        json_payload = {}

        if device_id is not None:
            json_payload["device_id"] = device_id

        self.seam.client.post("/devices/simulate/remove", json=json_payload)

        return None



from seam.types import AbstractSeam as Seam
from seam.routes.types import AbstractActionAttempts, ActionAttempt
from typing import Optional, Any, List, Dict, Union

import time


class SeamActionAttemptError(Exception):
    def __init__(self, message: str, action_attempt: ActionAttempt):
        super().__init__(message)
        self.name = self.__class__.__name__
        self.action_attempt = action_attempt


class SeamActionAttemptFailedError(SeamActionAttemptError):
    def __init__(self, action_attempt: ActionAttempt):
        super().__init__(action_attempt.error.message, action_attempt)
        self.name = self.__class__.__name__
        self.code = action_attempt.error.type


class SeamActionAttemptTimeoutError(SeamActionAttemptError):
    def __init__(self, action_attempt: ActionAttempt, timeout: str):
        message = f"Timed out waiting for action attempt after {timeout}s"
        super().__init__(message, action_attempt)
        self.name = self.__class__.__name__


class ActionAttempts(AbstractActionAttempts):
    seam: Seam

    def __init__(self, seam: Seam):
        self.seam = seam

    def get(
        self,
        *,
        action_attempt_id: str,
        wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]] = None,
    ) -> ActionAttempt:
        json_payload = {}

        if action_attempt_id is not None:
            json_payload["action_attempt_id"] = action_attempt_id

        res = self.seam.client.post("/action_attempts/get", json=json_payload)

        return self.seam.action_attempts.decide_and_wait(
            action_attempt=ActionAttempt.from_dict(res["action_attempt"]),
            wait_for_action_attempt=wait_for_action_attempt,
        )

    def list(self, *, action_attempt_ids: List[str]) -> List[ActionAttempt]:
        json_payload = {}

        if action_attempt_ids is not None:
            json_payload["action_attempt_ids"] = action_attempt_ids

        res = self.seam.client.post("/action_attempts/list", json=json_payload)

        return [ActionAttempt.from_dict(item) for item in res["action_attempts"]]

    def poll_until_ready(
        self,
        *,
        action_attempt_id: str,
        timeout: Optional[float] = 5.0,
        polling_interval: Optional[float] = 0.5,
    ) -> ActionAttempt:
        seam = self.seam
        time_waiting = 0.0

        action_attempt = seam.action_attempts.get(
            action_attempt_id=action_attempt_id, wait_for_action_attempt=False
        )

        while action_attempt.status == "pending":
            time.sleep(polling_interval)
            time_waiting += polling_interval

            if time_waiting > timeout:
                raise SeamActionAttemptTimeoutError(action_attempt, timeout)

            action_attempt = seam.action_attempts.get(
                action_attempt_id=action_attempt.action_attempt_id,
                wait_for_action_attempt=False,
            )

        if action_attempt.status == "failed":
            raise SeamActionAttemptFailedError(action_attempt)

        return action_attempt

    def decide_and_wait(
        self,
        *,
        action_attempt: ActionAttempt,
        wait_for_action_attempt: Optional[Union[bool, Dict[str, float]]] = None,
    ) -> ActionAttempt:
        wait_decision = (
            self.seam.wait_for_action_attempt
            if wait_for_action_attempt is None
            else wait_for_action_attempt
        )

        if wait_decision is True:
            return self.seam.action_attempts.poll_until_ready(
                action_attempt_id=action_attempt.action_attempt_id
            )
        if isinstance(wait_decision, dict):
            return self.seam.action_attempts.poll_until_ready(
                action_attempt_id=action_attempt.action_attempt_id,
                timeout=wait_decision.get("timeout", None),
                polling_interval=wait_decision.get("polling_interval", None),
            )

        return action_attempt
